Package: CyTOFpower
Type: Package
Title: Power analysis for CyTOF experiments
Version: 1.19.1
Authors@R: c(person("Anne-Maud", "Ferreira",
           email = "anne-maud.ferreira@epfedu.fr",
           role = c("cre", "aut"),
           comment = c(ORCID = "0000-0002-4749-746X")),
    person("Catherine", "Blish",
            email = "cblish@stanford.edu",
            role = c("aut")),
    person("Susan", "Holmes",
            email = "sph@stanford.edu",
            role = c("aut")))
Description: This package is a tool to predict the power of CyTOF 
    experiments in the context of differential state analyses. The package provides 
    a shiny app with two options to predict the power of an experiment: 
    i. generation of in-sicilico CyTOF data, using users input ii. browsing in a 
    grid of parameters for which the power was already precomputed.
biocViews: FlowCytometry, SingleCell, CellBiology, StatisticalMethod,
        Software
License: LGPL-3
Encoding: UTF-8
Depends: R (>= 4.1)
Imports: CytoGLMM, diffcyt, DT, dplyr, ggplot2, magrittr, methods,
        rlang, stats, shiny, shinyFeedback, shinyjs, shinyMatrix,
        SummarizedExperiment, tibble, tidyr
RoxygenNote: 7.1.2
VignetteBuilder: knitr
Suggests: testthat (>= 3.0.0), BiocStyle, knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/CyTOFpower
git_branch: devel
git_last_commit: b8a8306
git_last_commit_date: 2026-05-02
Repository: Bioconductor 3.24
Date/Publication: 2026-05-12
NeedsCompilation: no
Packaged: 2026-05-12 21:12:10 UTC; biocbuild
Author: Anne-Maud Ferreira [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-4749-746X>),
  Catherine Blish [aut],
  Susan Holmes [aut]
Maintainer: Anne-Maud Ferreira <anne-maud.ferreira@epfedu.fr>
