#' CyTOFpower: R-package to predict the power of a CyTOF experiment
#'
#' This package provides functions and a shiny app to predict the power of a
#' CyTOF experiment.
#'
#' @docType package
#' @name CyTOFpower
#'
#' @import methods
#' @import shiny
#'
#' @importFrom shinyjs useShinyjs
#' @importFrom shinyFeedback useShinyFeedback
#' @importFrom rlang .data
#' @importFrom magrittr "%>%"
#' @importFrom dplyr bind_rows
NULL
